.TH "qube-fndprg" 1 "April 3, 2021" "qube"
.SH NAME
qube-fndprg \- check than an executable program is available
.SH SYNOPSIS
\fBqube-fndprg\fR \fI[ -h | -V ] | [ -n | -p <program> ] [ -v ]\fR
.SH DESCRIPTION
\fBqube-fndprg\fR tries to run \fI<program>\fR and exits 0 on success. Otherwise
- in case \fI<program>\fR could not be executed - it exits 2 (ENOENT). If
\fI<program>\fR was not spezified \fBqube-fndprg\fR does fallback to use 'bash'.

The core intention of \fBqube-fndprg\fR is to determine if a required shell is
available, because a script requires this dedicated shell. Doing this inside a
script is error-prone, against \fBqube-fndprg\fR is much more failure tolerant
indepandant of the active/default shell. \fBqube\fR uses bourne shell compatible
scripts always - which will usually fail with (t)csh.

For shell detection \fBqube-fndprg\fR uses '-c set' as parameter of the checked
shell. This is a convention and shall work with _all_ UNIX* shells. Right now
this was choosen against the (cheaper?) alternative '-c echo -n'. 

\fBqube-fndprg\fR is not limited to shell detection. The options \fI\-n\fR and
\fI-p\fR shall qualify \fBqube-fndprg\fR to check for every executable binary.
However, the output of \fBqube-fndprg\fR have to be verified before used with a
script (see below).

\fBqube-fndprg\fR works like \fIwhich\fR in most cases. Thus it is comparable
but 

.SH OPTIONS
The \fIoptions\fR are a series of getopt-style short options. \fI<program>\fR
could be given as option argument (recommanded) or as positional argument.
.TP
.I \-h
Print a short help and exit.
.BR
.TP
.I \-n
Run \fI<program>\fR without any parameters. Usually \fI<program>\fR doesn't have
an option '--version'.
.BR
.TP
.I \-p
Run \fI<program>\fR with parmeter '--version'. This is especially useful for
interpreters like perl or python, otherwise they will go in interactive mode.
.BR
.TP
Both options \fI\-p\fR and \fI\-s\fR requires an argument.
.BR
.TP
.I \-v
Increase the verbosity level by +1 for each occurance. Higher levels include lower
ones. Levels:

  0: silent, no output at all (this is the default)
  1: print an error message on failure
  2: print output of \fI<program>\fR
.BR
.TP
.I \-V
Print version info and exit.
.BR
.SH FUTURE DIRECTIONS
\fBqube-fndprg\fR is another step to make UNIX* like system more comfortable usable.
Increasing usability and simplicity is an ongoing process.
.SH EXAMPLES
1) Check for 'bash' suppressing any output except an error occurs and change command
interpreter to bash (script mode):

.EX
    #!/usr/bin/env sh

    qube-fndprg -v || exit 2
    /usr/bin/env bash <<<EOF
    source ./configure.h
    EOF
.EE

Example for interactive mode:

2) Check for 'python' and print the output of 'python --version':

.EX
    $> qube-fndprg -vv -p python
.EE

.SH AUTHOR
Kai Peter (©2021-QYYYY)
.SH SEE ALSO
