#********************************************************************************
# qware build environment - qube-srcpkg                                         *
#                                                                               *
#  Author: Kai Peter ©2015-present                                              *
# Version: 2.0                                                                  *
# License: ISC                                                                  *
#                                                                               *
# Description: Create a source code release.                                    *
#********************************************************************************
#---------- begin (standard) qware configuration section ------------------------
VERSION=`cat $0 | grep -m 1 'Version' | cut -d\   -f3`       # get version number
ME=`basename $0`

LOGDATE=`date +'%b %d %H:%M:%S'`
LOGFILE="QLOGDIR/${ME}.log"
DATESTR=`date +%Y%m%d`
LIBDIR="QLIBDIR"

[ $DBG ] || DBG=0                                # set debug level (dafault: off)

error() { printf "\033[1mError: %s\033[0m\n" "$1"; exit 1; }
warning() { printf "\033[93mWarning: %s\033[0m\n" "$1"; return; }
#---------- end qware configuration section -------------------------------------
#getSrcDir() {
#  return
#}

prepSources() {     # check and prepare the sources *****************************
  TMPDIR=`mktemp -d tmp.XXXXXXXXXX` || error "Unable to create temp. directory!"
  cd "$SRCDIR" || error "Unable to change into $SRCDIR"
  echo -n "Preparing sources ... "
  cd $OLDPWD
  # set includes and excludes for rsync
  [ -f .qube/pkginclude ] && INCLUDE="--include-from .qube/pkginclude"
  [ -f .qube/pkgexclude ] && EXCLUDE="--include-from .qube/pkginclude"
  # by convention we use a .gitignore file in the top level source dir
  [ -f $SRCDIR/.gitignore ] && EXCLUDE=" --exclude-from $SRCDIR/.gitignore"
  rsync -Lr $SRCDIR/* $TMPDIR $INCLUDE $EXCLUDE
  echo "done"
}
#********************************************************************************
mkArchive() {       # create the archive ****************************************
  ARCHIVE="$TARGET-$VERSION".tgz
  # some time the VERSION file contains the target name, sometimes not
#  echo $ARCHIVE | grep $TARGET >DEVNULL || ARCHIVE="$TARGET-$VERSION".tgz
#  ARCHIVE=$TARGET.tgz
  echo -n "Creating archive ... "
  tar -cvzf $ARCHIVE $TMPDIR --transform s/"$TMPDIR"/"$TARGET"/ 2>&1>DEVNULL
  sha256sum $ARCHIVE > $ARCHIVE.sha256
  echo "done"

  if [ "$RELEASESDIR" ] ; then
    [ ! -d "$RELEASESDIR" ] && mkdir "$RELEASESDIR"
    mv $ARCHIVE* $RELEASESDIR
  fi
  [ "$DBG" -lt 1 ] && rm -fr "$TMPDIR"              # clean up on low debug level
}
getTopQubeDir() {
  CURDIR=`pwd`
  until [ -d .qube ] ; do
    cd ..
    [ `pwd` = "/" ] && error "Not a qube project up to dir '/'."
    df | grep -q `pwd` && error "Not a qube project up to mountpoint `pwd`."
  done
  TOPQUBEDIR=`pwd`
}
#***** begin processing *********************************************************
getTopQubeDir            # directory where to start

# the config file HAVE TO exist _in this location_
CFGFILE=".qube/config"
[ ! -f "$CFGFILE" ] && error "No config file found!" || source "$CFGFILE"

# some default settings (fallback, usually set in config file)
[ ! $SRCDIR ] && SRCDIR="src"
[ ! -d "$SRCDIR" ] && error "Unable to determine sources directory!"

if [ ! "$TARGET" ] ; then
  [ -f $SRCDIR/PKGNAME.dist ] && TARGET=`head -1 "$SRCDIR/PKGNAME.dist"`; fi
[ ! $TARGET ] && error "Unable to determine a target name - aborting!"

# optional
[ ! -f $SRCDIR/LICENSE ] && warning "No file LICENSE found."
[ ! -f $SRCDIR/VERSION ] && warning "No file VERSION found - using today's date." && \
    VERSION="$DATESTR" || VERSION=`head -1 $SRCDIR/VERSION`

rm -rf tmp.*        # remove old TMPDIR's
echo "Attempting to get changelog status ... "
$LIBDIR/qube-chglog
prepSources         # prepare the sources
mkArchive           # create archive
echo "Successful created source code release $ARCHIVE!"
#exit 0
