#********************************************************************************
# qware build environment - control tool                                        *
#                                                                               *
#  Author: Kai Peter, ©2020-YYYY                                                *
# Version: 0.1                                                                  *
# License: ISC                                                                  *
#                                                                               *
# Description: The qube command line control tool.                              *
#********************************************************************************
# standard configuration section
ME=`basename "$0"`
VERSION=`cat "$0" | grep -m 1 'Version' | cut -d\   -f3`     # get version number
declare -i DBG=0                                 # set debug level (default: off)
#LOGDATE=`date +'%b %d %H:%M:%S'`
#LOGFILE="QLOGDIR/${ME}.log"
CFGFILE="qubepkg.conf"
#--------------------------------------------------------------------------------

# test: detect libdir automatically
#LOC=`readlink $0`
#[ -n "$LOC" ] chdir `dirname $LOC`

LIBDIR="QLIBDIR"

showVersion() { echo "$ME-$VERSION"; return; }

showHelp() {
  echo -e "Usage: $ME <module> [ <options> ]"
#  echo -e "Usage: $ME [ <options> ] <action> [ <name> ]"
  echo -e ""
  echo -e "Modules:"
  echo -e "   binpkg   \t create a binary distributable package"
  echo -e "   chglog   \t update the changelog from git repo"
  echo -e "   newprj   \t create a new project <name>"
  echo -e "   srcpkg   \t create a source code release"
  echo -e "   upload   \t upload and publish an archive"
  echo -e "   help     \t show general help screen"
  echo -e ""
  echo -e "Options:"
  echo -e "   -f | --configfile  \t qube config file to use (new projects only ***)"
  echo -e "   -h | --help        \t show help for <module>"
  echo -e "   -n | --name        \t name of a new project (with 'newprj' only ***)"
  echo -e "   -p | --path        \t an abolute or relative path for a project"
  echo -e "   -V | --version     \t show version info and exit"
  echo -e ""
  echo -e "Short options could be replaced by long options."
  exit 0
}

hlpModules() {
  echo "Usage $ME $MODULE <options>" ; echo; echo -e "Options:"
  if [ "$MODULE" = "newprj" ] ; then
    echo -e "   -f|name) \t an alternative config file used by $ME"
    echo -e "   -n|name) \t the name of the new project"
    echo -e "   -p|path) \t the path where a new project <name> will be created"
    return
  fi
  echo -e "   -p|path) \t the path of an existing qube project to work on"
}
error() {
  printf "\033[1merror: \033[0m"
  case $1 in
    1) echo "Couldn't find module qube-$MODULE!";;
    2) echo "Invalid or missing argument for option '`echo $ARG | cut -d= -f1`'!";;
#    3) echo "Invalid parameter for '-p'!";;
    6) echo "Not a qube project folder: $FLAG_P";;
    5) echo "${FLAG_P%/}/$FLAG_NAME is a qube project already!";;
    *) echo "unknown error!";;
  esac
  exit $1
}

# validate that required arguments for options are not invalid in general *******
chkOptarg() { ( echo "$1" | grep -q ^'-' || [ -z "$1" ] ) && error 2; }



newProject() {  # 'new project' is a bit more special with its options **********
#echo running newprj
  local NAME="$FLAG_N"

  #
  [ `basename "$FLAG_P"` = "$NAME" ] && FLAG_P=`dirname "$FLAG_P"`


  [ -f "$FLAG_P/$NAME/$CFGFILE" ] && error 5     # --> is an qube project already
  # set and insert a potential path option according to qube-newprj' requirements
  [ -n "$FLAG_P" ] && ACTION="$ACTION -p $FLAG_P"

  # adjust an alternative config file option  along the qube-newprj' requirements
  [ -n "$FLAG_C" ] && ACTION="$ACTION -c $FLAG_C"
  ACTION="$ACTION $NAME"
}

chkAction() {   # 'standard actions' for an existing qube project ***************
  # check the given path is a valid qube directory and change into it
  [ -f "$FLAG_P/$CFGFILE" ] && cd "$FLAG_P" || error 6
}

#***** begin processing *********************************************************
declare -r P=$@                          # a string with all command line options
declare -r O=($P)                        # an array with all the arguments
unset ACTION
unset MODULE
#echo A: "$#"

# get the module (action)
case "$1" in
        B|binpkg) MODULE="binpkg";;
        L|chglog) MODULE="chglog";;
        N|newprj) MODULE="newprj";;
        S|srcpkg) MODULE="srcpkg";;
        U|upload) MODULE="upload";;
  -h|--help|help) showVersion; showHelp;;
    -V|--version) showVersion; exit 0;;
               *) echo -e "\033[1mInvalid module: '$1'!\033[0m\n"; showHelp;;
esac


declare -i idx=0                         # loop index (counter) 
for ARG in $P ; do
  idx=`expr $idx + 1`      # increment the index
#echo 1: ${O[idx]}
  case "$ARG" in
    # internal actions
       -h|--help) hlpModules ; exit 0;;
#    -V|--version) echo "$ME-$VERSION" ; exit 0;;  # print the module version
    # options (flags)
    # FLAG_P: path
    # FLAG_C: config file
    # FLAG_N: name for a new project
 -f|--configfile) FLAG_C="${O[idx]}"; chkOptarg "$FLAG_C";;      # config file (w/ qube-newprj)
       -n|--name) FLAG_N="${O[idx]}"; chkOptarg "$FLAG_N";;      # a path (usage depends on action)
       -p|--path) FLAG_P="${O[idx]}"; chkOptarg "$FLAG_P";;      # a path (usage depends on action)
              -v) DBG=$(($DBG+1)) ; [ $DBG -gt 9 ] && DBG=9;;
    # long option with '='
  --configfile=*) FLAG_C=`echo $ARG | cut -d= -f2`; chkOptarg "$FLAG_C";;
        --name=*) FLAG_N=`echo $ARG | cut -d= -f2`; chkOptarg "$FLAG_N";;
        --path=*) FLAG_P=`echo $ARG | cut -d= -f2`; chkOptarg "$FLAG_P";;
             -v*);; # todo: increase DBG level by number of "v"'s --> nice to have :)
  esac
done
#
ACTION="$LIBDIR/qube-$MODULE" #; [ -f "$ACTION" ] || error 1
#echo $ACTION
# select between 'new project' and the other actions
[ "$ACTION" != "$LIBDIR/qube-newprj" ] && chkAction || newProject

echo $ACTION
#exec "$ACTION"
exit 0
